 /*
 * file      : src/edit_year.c
 * project   : tm
 * with      : Gtk-2
 *
 * copyright : (C) 2012 by Claude BULIN
 *
 * TM - Edition des heures de travail / Edition of working hours
 * GNU General Public License
 * 
 * This file is part of TM.
 * 
 * TM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * at your option) any later version.
 * 
 * TM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with TM.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
	#define gettext_noop(x) (x) 
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_tm/lib.h"
#endif

#include "edit_year.h"
#include "global.h"
#include "win_new_year.h"
#include "configuser.h"
#include "callbacks.h"
#include "win_info.h"
#include "draw.h"


typedef struct {
	gint		IndiceJour;		// Indice du jour: 0 .. NbrJoursAnnee
	gint		ThisDay;		// 1=LUNDI .. 7=DIMANCHE
	gint		Day;			// 
	gint		Month;			// 
	gint		MatinDebut;		// Debut heure de travail en minutes
	gint		MatinFin;		// Fin heure travail en minutes
	gint		SoirDebut;		// Debut heure de repas en minutes
	gint		SoirFin;		// Fin heure de repas en minutes
	gint		WorkingTime;		// Heures de la semaine stockees le dimanche
	gint		OverTime;		// Heures superieures a HEURES_NORMALES_SEMAINE stockees le dimanche
	gchar		*Comment;		// Commentaire
} VAR_EDIT_DAY;

typedef struct {
	gboolean	BoolReady;		// It's ready to edit
	gboolean	BoolEdit;		// TRUE IF edit
	gboolean	BoolChanged;		// If TRUE then CALL: EditYear_spinbutton_time_value_changed()
	GtkWidget	*AdrCalendar;		// Adresse Calendar
	guint		YearEdit;		// Annee actuelle en edition
	gint		NbrJoursAnnee;		// Total jours par an
	gint		DayOfYear;		// Jour de l'annee ou -1
	gchar		**Larrbuf;		// Buffer file datas
	VAR_EDIT_DAY	*Day;			// (VAR_EDIT_DAY *)g_malloc0( sizeof(VAR_EDIT_DAY) * VarEdit.NbrJoursAnnee )
	VAR_EDIT_DAY	*DayAdjust;		// 
} VAR_EDIT;

#define MAX_DAY_ADJUST 10

VAR_EDIT VarEdit = {
	FALSE,					// It's ready to edit
	FALSE,					// TRUE IF edit
	TRUE,					// If TRUE then CALL: EditYear_spinbutton_time_value_changed()
	NULL,					// Adresse Calendar
	0,					// Annee actuelle en edition
	0,					// Total jours par an
	-1,					// Jour de l'annee ou -1
	NULL,					// Buffer file datas
	NULL,					// (VAR_EDIT_DAY *)g_malloc0( sizeof(VAR_EDIT_DAY) * VarEdit.NbrJoursAnnee )
	NULL					// VAR_EDIT_DAY	*DayAdjust
	};


gchar	*MMonth[]={
	gettext_noop("Janvier"),
	gettext_noop("Fevrier"),
	gettext_noop("Mars"),
	gettext_noop("Avril"),
	gettext_noop("Mai"),
	gettext_noop("Juin"),
	gettext_noop("Juillet"),
	gettext_noop("Aout"),
	gettext_noop("Septembre"),
	gettext_noop("Octobre"),
	gettext_noop("Novembre"),
	gettext_noop("Decembre")
	};
	
gchar	*DDay[]={
	gettext_noop("Lundi"),
	gettext_noop("Mardi"),
	gettext_noop("Mercredi"),
	gettext_noop("Jeudi"),
	gettext_noop("Vendredi"),
	gettext_noop("Samedi"),
	gettext_noop("Dimanche")
	};



// 
// 
gchar *EditYear_get_day( gint p_day )
{
	if( p_day < 0 || p_day > 6 ) p_day = 0;
	return( (gchar *)DDay[ p_day ] );
}
// 
// 
gchar *EditYear_get_month( gint p_month )
{
	if( p_month < 0 || p_month > 11 ) p_month = 0;
	return( (gchar *)MMonth[ p_month ] );
}
// 1=Lundi ... 7=Dimanche
// 
gint EditYear_get_weekday( guint p_day, guint p_month, guint p_year )
{
	gint	Day = 1;
	GDate	*DateHeap;
	
	DateHeap = g_date_new ();
	g_date_set_year( DateHeap, p_year );
	g_date_set_month( DateHeap, p_month );
	g_date_set_day( DateHeap, p_day );
	Day = g_date_get_weekday( DateHeap );
	g_date_free( DateHeap );
	
	return( Day );
}
// Retourne DEBUT et FIN de p_month et p_year
// 
void EditYear_get_base_days_in_month( guint p_month, guint p_year, gint *p_begin, gint *p_end )
{
	GDate	*DateHeap;
	
	// FIXER ANNEE MOIS JOUR
	DateHeap = g_date_new ();
	g_date_set_year( DateHeap, p_year );
	g_date_set_month( DateHeap, p_month +1 );
	g_date_set_day( DateHeap, 1 );
	
	*p_begin = g_date_get_day_of_year( DateHeap );
	*p_end = *p_begin + g_date_get_days_in_month( p_month +1, p_year );
		
	g_date_free( DateHeap );
}
// STOCKAGE LE DIMANCHE
// 	HEURES SEMAINE
//	HEURES SUPERIEURE A HEURES_NORMALES_SEMAINE
void EditYear_get_overtime( void )
{
	gint	Day;
	gint	S = 0;
	
	// INIT
	for( Day = 0; Day < VarEdit.NbrJoursAnnee; Day ++ ) {
		VarEdit.Day[ Day ].WorkingTime = 0;
		VarEdit.Day[ Day ].OverTime    = 0;
	}
	// 
	for( Day = 0; Day < VarEdit.NbrJoursAnnee; Day ++ ) {
		
		// HEURES DE LA JOURNEE
		S += (VarEdit.Day[ Day ].MatinFin - VarEdit.Day[ Day ].MatinDebut) + (VarEdit.Day[ Day ].SoirFin - VarEdit.Day[ Day ].SoirDebut);
		
		if( VarEdit.Day[ Day ].ThisDay == 7 ) {
			// HEURES DE LA SEMAINE
			VarEdit.Day[ Day ].WorkingTime = S;
			// SEMAINE MOINS HEURES_NORMALES_SEMAINE
			// if( S >= 2220 )
			// 	VarEdit.Day[ Day ].OverTime = S - 2220;
			if( S >= Config.TimeByWeek )
				VarEdit.Day[ Day ].OverTime = S - Config.TimeByWeek;
			// RE-INIT
			S = 0;
		}
	}
}
// 
// NUMERO DE LA SEMAINE POUR UNE DATE
// 
// guint p_day    1 .. 31
// guint p_month  1 .. 12
// guint p_year   0 .. xxx	VarEdit.YearEdit
// 
guint EditYear_get_weeks( guint p_day, guint p_month, guint p_year )
{
	GDate	*DateHeap;
	guint	RetDay;
	
	DateHeap = g_date_new ();
	g_date_set_year( DateHeap, p_year );
	g_date_set_month( DateHeap, p_month );
	g_date_set_day( DateHeap, p_day );
	RetDay = g_date_get_iso8601_week_of_year( DateHeap );
	// g_print( "ANNEE %d\n", p_year );
	// g_print( "g_date_get_iso8601_week_of_year() = %d\n", NBR);
	g_date_free( DateHeap );
	
	return( RetDay );

}
// 
// NOMBRE DE JOUR DU MOIS p_month DE L'ANNEE p_year
// 
guint EditYear_get_days_in_month( guint p_month, guint p_year )
{
	GDate	*DateHeap;
	guint	NbrMonth;
	
	DateHeap = g_date_new ();
	g_date_set_year( DateHeap, p_year );
	g_date_set_month( DateHeap, p_month );
	g_date_set_day( DateHeap, 1 );
	NbrMonth = g_date_get_days_in_month( p_month, p_year );
	g_date_free( DateHeap );
	
	return( NbrMonth );

}
// 
// RETOURNE LE MOIS EN COURS
// 
guint EditYear_get_int_month( void )
{
	guint 	year, month, Dday;
	gtk_calendar_get_date( GTK_CALENDAR(VarEdit.AdrCalendar), &year, &month, &Dday );
	return( month );
}
// 
// DESSIN DE LA PAGE EN COURS
// 
void EditYear_draw_page( void )
{
	gchar	*Str = NULL;
	gchar	*StrMonth = NULL;
	gint	DayBegin, DayEnd;
	guint 	year, month, day, Dday;
	gint	H, M, T = 0;
	gdouble	LocateX = 15;
	gdouble	LocateY = 20;
	TYPE_COLOR	TypeColor = COLOR_BLACK;
#define LINE_PLUS 5
#define LINE_SIZE 10
	// INIT STRUCT
	if( FALSE == VarEdit.BoolReady ) return;
	
	gtk_calendar_get_date( GTK_CALENDAR(VarEdit.AdrCalendar), &year, &month, &Dday );
	if( VarEdit.YearEdit != year ) return;
	
	EditYear_get_base_days_in_month( month, year, &DayBegin, &DayEnd );
	DayBegin --;
	DayEnd --;
	
	StrMonth = libutils_string_toupper( EditYear_get_month( month ));
	Str = g_strdup_printf( "%s %d", StrMonth, year );
	draw_str( LocateX, LocateY, COLOR_BLACK, LINE_SIZE, Str );
	g_free( Str );
	Str = NULL;
	g_free( StrMonth );
	StrMonth = NULL;
	
	LocateY += LINE_PLUS;
	LocateY += LINE_PLUS;
	LocateY += LINE_PLUS;

	// CALCUL HEURES PAR SEMAINE ET HEURES SUPERIEURES A HEURES_NORMALES_SEMAINE
	EditYear_get_overtime();
	
	for( day = 1; DayBegin < DayEnd; DayBegin ++, day ++ ) {
		
		if( TRUE == draw_is_save_to_file() ) {
			TypeColor = COLOR_BLACK;
		}
		else {
			// INDIQUE SI JOURNEE ACTUELLE
			if( day == Dday ) {
				TypeColor = COLOR_RED;
				draw_str( LocateX -6, LocateY + LINE_PLUS, TypeColor, LINE_SIZE, "[*]" );
			}
			else {
				TypeColor = COLOR_BLACK;
			}
		}
		
		T += (VarEdit.Day[ DayBegin ].MatinFin - VarEdit.Day[ DayBegin ].MatinDebut) + (VarEdit.Day[ DayBegin ].SoirFin - VarEdit.Day[ DayBegin ].SoirDebut);
		H  = ((VarEdit.Day[ DayBegin ].MatinFin - VarEdit.Day[ DayBegin ].MatinDebut) + (VarEdit.Day[ DayBegin ].SoirFin - VarEdit.Day[ DayBegin ].SoirDebut)) / 60;
		M  = ((VarEdit.Day[ DayBegin ].MatinFin - VarEdit.Day[ DayBegin ].MatinDebut) + (VarEdit.Day[ DayBegin ].SoirFin - VarEdit.Day[ DayBegin ].SoirDebut)) % 60;
		
		Str = g_strdup_printf( " %02d    %s",
			EditYear_get_weeks( VarEdit.Day[ DayBegin ].Day, VarEdit.Day[ DayBegin ].Month, VarEdit.YearEdit ),
			EditYear_get_day( VarEdit.Day[ DayBegin ].ThisDay -1 )
			);
		LocateY += LINE_PLUS;
		draw_str( LocateX, LocateY, TypeColor, LINE_SIZE, Str );
		g_free( Str );
		Str = NULL;
		
		Str = g_strdup_printf( "%02d     %02d:%02d  %02d:%02d     %02d:%02d  %02d:%02d     %02d:%02d  %s",
			day,
			
			VarEdit.Day[ DayBegin ].MatinDebut / 60,
			VarEdit.Day[ DayBegin ].MatinDebut % 60,
			
			VarEdit.Day[ DayBegin ].MatinFin / 60,
			VarEdit.Day[ DayBegin ].MatinFin % 60,
			
			VarEdit.Day[ DayBegin ].SoirDebut / 60,
			VarEdit.Day[ DayBegin ].SoirDebut % 60,
			
			VarEdit.Day[ DayBegin ].SoirFin / 60,
			VarEdit.Day[ DayBegin ].SoirFin % 60,
			
			H, M,
						
			VarEdit.Day[ DayBegin ].Comment != NULL ? VarEdit.Day[ DayBegin ].Comment : ""
			);
		
		// if( VarEdit.Day[ DayBegin ].ThisDay >= 6 ) WinVte_set_color_begin( "CYAN" );
		draw_str( LocateX + 31, LocateY, TypeColor, LINE_SIZE, Str );
		// if( VarEdit.Day[ DayBegin ].ThisDay >= 6 ) WinVte_set_color_end();
		
		g_free( Str );
		Str = NULL;
		
		if( VarEdit.Day[ DayBegin ].ThisDay == 7 ) {
			// WinVte_set_color_begin( "YELLOW" );
			// AFFICHE LE NUMERO DE SEMAINE
			// Str = g_strdup_printf( "             [ semaine ] %02d\n", EditYear_get_weeks( VarEdit.Day[ DayBegin ].Day, VarEdit.Day[ DayBegin ].Month, VarEdit.YearEdit ));
			// WinVte_window_write_va( Str, NULL );
			// g_free( Str );
			// Str = NULL;
			// HEURES DE LA SEMAINE
			Str = g_strdup_printf( "heures:" );
			LocateY += LINE_PLUS;
			draw_str( LocateX +30, LocateY, COLOR_BLACK, LINE_SIZE, Str );
			g_free( Str );
			Str = NULL;
			
			Str = g_strdup_printf( "%02dh%02d", VarEdit.Day[ DayBegin ].WorkingTime / 60, VarEdit.Day[ DayBegin ].WorkingTime % 60 );
			draw_str( LocateX +45, LocateY, COLOR_BLACK, LINE_SIZE, Str );
			g_free( Str );
			Str = NULL;
			// HEURES SUPERIEURE A HEURES_NORMALES_SEMAINE
			if( VarEdit.Day[ DayBegin ].OverTime > 0 ) {
				Str = g_strdup_printf( "sup.:" );
				LocateY += LINE_PLUS;
				draw_str( LocateX +30, LocateY, COLOR_BLACK, LINE_SIZE, Str );
				g_free( Str );
				Str = NULL;
				
				Str = g_strdup_printf( "%02dh%02d", VarEdit.Day[ DayBegin ].OverTime / 60, VarEdit.Day[ DayBegin ].OverTime % 60 );
				// WinVte_window_write_va( Str, NULL );
				draw_str( LocateX +45, LocateY, COLOR_BLACK, LINE_SIZE, Str );
				g_free( Str );
				Str = NULL;
			}
			// WinVte_set_color_end();
			LocateY += LINE_PLUS;
		}
	}
	
	// WinVte_set_color_begin( "YELLOW" );
	// WinVte_window_write_va( "             [ Total heures du mois ] ", NULL );
	// LocateY += LINE_PLUS;
	// draw_str( LocateX, LocateY, COLOR_BLACK, LINE_SIZE, "             [ Total heures du mois ] " );
	Str = g_strdup_printf(  "Total heures du mois: %02dh%02d", T / 60, T % 60 );
	// WinVte_window_write_va( Str, NULL );
	// WinVte_set_color_end();
	LocateY += LINE_PLUS;
	LocateY += LINE_PLUS;
	draw_str( LocateX +30, LocateY, COLOR_BLACK, LINE_SIZE, Str );
	g_free( Str );
	Str = NULL;
}
// 
// 
void EditYear_set_flag_buttons( void )
{
	guint 	year, month, day;
	
	gtk_calendar_get_date( GTK_CALENDAR(VarEdit.AdrCalendar), &year, &month, &day );
	
	if( TRUE == VarEdit.BoolReady && year == VarEdit.YearEdit ) {
		gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("vbox_saisie")), TRUE );
	}
	else {
		gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("vbox_saisie")), FALSE );		
	}
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("button_export_year")), VarEdit.BoolReady );	
	gtk_widget_set_sensitive( GTK_WIDGET(GLADE_GET_OBJECT("button_init_base_year")), VarEdit.BoolReady );	

	draw_year();
}
// 
// 
void EditYear_remove_struct_year( void )
{
	gint	Day;
	
	if( NULL != VarEdit.Day ) {
		// REMOVE Comment
		for( Day = 0; Day < VarEdit.NbrJoursAnnee; Day ++ ) {
			if( NULL != VarEdit.Day[ Day ].Comment );
		}
		// REMOVE Struct
		g_free( VarEdit.Day );
		VarEdit.Day = NULL;
	}
}
// 
// 
void EditYear_clear_year( void )
{
	EditYear_remove_struct_year();
	VarEdit.BoolReady = FALSE;
	VarEdit.BoolEdit  = FALSE;
	draw_year();
	
}
// 
// 
void EditYear_remove_struct_all( void )
{
	gint	Day;

	EditYear_remove_struct_year();

	if( NULL != VarEdit.DayAdjust ) {
	
		// REMOVE Comment
		for( Day = 0; Day < MAX_DAY_ADJUST; Day ++ ) {
			if( NULL != VarEdit.DayAdjust[ Day ].Comment );
		}
		// REMOVE Struct
		g_free( VarEdit.DayAdjust );
		VarEdit.DayAdjust = NULL;
	}
}
// 
// 
gint EditYear_get_day_of_year( guint p_year )
{
	gint	day_of_year = 0;
	GDate	*DateHeap;
	
	DateHeap = g_date_new ();
	g_date_set_year( DateHeap, p_year );
	g_date_set_month( DateHeap, 12 );
	g_date_set_day( DateHeap, 31 );
	day_of_year = g_date_get_day_of_year( DateHeap );
	g_date_free( DateHeap );
	return( day_of_year );
}
// 
// AFFICHE LES HEURES SUP. PAR SEMANE POUR UNE ANNEE
// 
void EditYear_put_weeks( void )
{
	gint	Day = 0;
	gint	W;
	
	// CALCUL HEURES PAR SEMAINE ET HEURES SUPERIEURES A HEURES_NORMALES_SEMAINE
	EditYear_get_overtime();
	
	// ENTETE
	g_print( "\n" );
	g_print( "S :   L. M. M. J. V. S. D.\n" );
	
	// AFFICHE LE NUMERO DE SEMAINE
	g_print( "%02d:   ", EditYear_get_weeks( VarEdit.Day[ Day ].Day, VarEdit.Day[ Day ].Month, VarEdit.YearEdit ));
	
	// SI DEBUT DE SEMAINE DIFFERENT DE LUNDI
	for( W = 1; W < VarEdit.Day[ Day ].ThisDay; W ++ ) {
		g_print( "   " );
	}
	
	for( Day = 0; Day < VarEdit.NbrJoursAnnee; Day ++ ) {
		
		// SI LUNDI
		if( 1 == VarEdit.Day[ Day ].ThisDay ) {
			// AFFICHE LE NUMERO DE SEMAINE
			g_print( "%02d:   ", EditYear_get_weeks( VarEdit.Day[ Day ].Day, VarEdit.Day[ Day ].Month, VarEdit.YearEdit ));
		}
		
		// AFFICHE LE JOUR
		g_print( "%02d ", VarEdit.Day[ Day ].Day );
		
		// SI DIMANCHE
		if( 7 == VarEdit.Day[ Day ].ThisDay ) {
			// HEURES SUPERIEURE A HEURES_NORMALES_SEMAINE
			if( VarEdit.Day[ Day ].OverTime > 0 ) {
				g_print( " [ heures sups ] %02dh%02d", VarEdit.Day[ Day ].OverTime / 60, VarEdit.Day[ Day ].OverTime % 60 );
			}
			// LIGNE SUIVANTE
			g_print( "\n" );
		}
	}
}
// 
// 
void EditYear_alloc_struct( void )
{
	gint	weekday;
	gint	Day;
	gint	Month;
	gint	NbrMonth;
	gint	Indice;
	
	EditYear_remove_struct_year();
	
	// TOTAL NUMBER DAYS OF YEAR	
	VarEdit.NbrJoursAnnee = EditYear_get_day_of_year( VarEdit.YearEdit );
	
	// ALOCATE STRUCT
	VarEdit.Day = (VAR_EDIT_DAY *)g_malloc0( sizeof(VAR_EDIT_DAY) * (VarEdit.NbrJoursAnnee +10) );
	
	weekday = EditYear_get_weekday( 1, 1, VarEdit.YearEdit );
	
	// INIT STRUCT
	// Day = 0 .. VarEdit.NbrJoursAnnee
	Day = 0;
	// POUR TOUS LES MOIS
	for( Month = 1; Month <= 12; Month ++ ) {
		// POUR TOUS LES JOURS DU MOIS
		NbrMonth = EditYear_get_days_in_month( Month, VarEdit.YearEdit );
		for( Indice = 1; Indice <= NbrMonth; Indice ++ ) {
			
			if( weekday > 7 ) weekday = 1;
		
			VarEdit.Day[ Day ].IndiceJour       = Day;	// 0 .. VarEdit.NbrJoursAnnee -1
			VarEdit.Day[ Day ].ThisDay          = weekday;	// 1=LUNDI .. 7=DIMANCHE
			VarEdit.Day[ Day ].Day              = Indice;	// 1 .. 31
			VarEdit.Day[ Day ].Month            = Month;	// 1=JANVIER .. 12=DECEMBRE
			VarEdit.Day[ Day ].WorkingTime      = 0;	// Heures de la semaine stockees le dimanche
			VarEdit.Day[ Day ].OverTime         = 0;	// Heures superieures a HEURES_NORMALES_SEMAINE stockees le dimanche
			
			if( weekday >= 6 ) {
				VarEdit.Day[ Day ].MatinDebut       = 0;
				VarEdit.Day[ Day ].MatinFin         = 0;
				VarEdit.Day[ Day ].SoirDebut        = 0;
				VarEdit.Day[ Day ].SoirFin          = 0;
				VarEdit.Day[ Day ].Comment          = g_strdup( "Week End" );
			}
			else {
				VarEdit.Day[ Day ].MatinDebut       = 450;
				VarEdit.Day[ Day ].MatinFin         = 690;
				VarEdit.Day[ Day ].SoirDebut        = 720;
				VarEdit.Day[ Day ].SoirFin          = 975;
				VarEdit.Day[ Day ].Comment          = g_strdup( "" );
			}
			weekday ++;
			Day ++;
		}
	}
}
// 
// 
void EditYear_save_struct( void )
{
	FILE	*fp = NULL;
	gint	Day;
	
	if( FALSE == VarEdit.BoolEdit ) return;
	if( NULL == VarEdit.Day ) return;
	
	if( NULL != (fp = fopen( Config.PathNameFileBase, "w" ))) {
		
		// PRINT_FUNC_LF();
		
		// AM	AVANT Midi
		// PM	APRES Midi
		fprintf (fp, "# \n" );
		fprintf (fp, "# Datas for tm program\n" );
		fprintf (fp, "# \n" );
		fprintf (fp, "# copyright : (C) 2012 by Claude BULIN\n" );
		fprintf (fp, "# Time Management - Edition des heures de travail / Edition of working hours\n" );
		fprintf (fp, "# GNU General Public License\n" );
		fprintf (fp, "# \n" );
		fprintf (fp, "# DAY AM_TIME_BEGIN_IN_MINUTES AM_TIME_END_IN_MINUTES PM_TIME_BEGIN_IN_MINUTES PM_TIME_END_IN_MINUTES COMMENT\n" );
		fprintf (fp, "# \n" );
		fprintf (fp, "\n" );
		fprintf (fp, "YEAR %d\n", VarEdit.YearEdit );
		fprintf (fp, "NUMBER_DAY %d\n", VarEdit.NbrJoursAnnee );
		fprintf (fp, "\n" );
		
		// INIT STRUCT
		
		// DAY 1 190 690 720 975 Comment
		// DAY 2 191 690 720 975 Comment

		for( Day = 0; Day < VarEdit.NbrJoursAnnee; Day ++ ) {
			
			fprintf (fp, "DAY " );
			fprintf (fp, "%d ", VarEdit.Day[ Day ].IndiceJour );
			fprintf (fp, "%d ", VarEdit.Day[ Day ].MatinDebut );
			fprintf (fp, "%d ", VarEdit.Day[ Day ].MatinFin );
			fprintf (fp, "%d ", VarEdit.Day[ Day ].SoirDebut );
			fprintf (fp, "%d ", VarEdit.Day[ Day ].SoirFin );
			fprintf (fp, "%s",  VarEdit.Day[ Day ].Comment != NULL ? VarEdit.Day[ Day ].Comment : "" );
			fprintf (fp, "\n" );
		}
		fclose( fp );
		VarEdit.BoolEdit = FALSE;
		EditYear_set_flag_buttons();
	}
}
// 
// 
gboolean EditYear_read_file( gchar *p_PathNameFileBase )
{
	FILE		*fp = NULL;
	size_t 		SizeFile;
	gchar		*Buffer = NULL;
	gint		Cpt;
	gchar		*Ptr = NULL;
	gboolean	BoolAllocStruct = FALSE;
	gint		Day;
	gchar		**LarrbufLine = NULL;
	
	VarEdit.Larrbuf = NULL;
	if(( SizeFile = libutils_get_size_file( p_PathNameFileBase )) > 0 ) {
		if( NULL != ( fp = fopen( p_PathNameFileBase, "r" ))) {
			Buffer = (gchar *)g_malloc0( sizeof(gchar) * ( SizeFile +10 ));
			SizeFile = fread( Buffer, SizeFile, 1, fp );
			fclose( fp );
			VarEdit.Larrbuf = g_strsplit( Buffer, "\n", 0 );
			g_free( Buffer );
			Buffer = NULL;
		}
	}
	
	// VERIFY FILE FOR TM
	for( Cpt = 0; VarEdit.Larrbuf[ Cpt ]; Cpt++ ) {
		
		if( strncmp( VarEdit.Larrbuf[ Cpt ], "# Datas for tm program", 22 ) == 0 ) {
			BoolAllocStruct = TRUE;
			break;
		}
	}
	
	if( FALSE == BoolAllocStruct ) {
		
		wininfo_create ( WindMain,
			_("Erreur choix de fichier !"),
			_("   Ce fichier ne correspond pas     "),
			  "\n",
			_("   a l'application"),
			  "\n",
			_("   Time Management"),
			  "\n\n",
			_("   Veuillez recommencer."),
			  "\n",
			  "");
	
		g_strfreev( VarEdit.Larrbuf );
		VarEdit.Larrbuf = NULL;
		return( FALSE );
	}
	
	// READ: YEAR
	// ALLOCATE STRUCT WITH: NUMBER_DAY
	for( Cpt = 0; VarEdit.Larrbuf[ Cpt ]; Cpt++ ) {
		
		// Si commentaire ou ligne vide alors ligne suivante
		if( *VarEdit.Larrbuf[ Cpt ] == '#' || *VarEdit.Larrbuf[ Cpt ] == '\0') continue;
		
		if( strncmp( VarEdit.Larrbuf[ Cpt ], "YEAR", 4 ) == 0 ) {
			Ptr = VarEdit.Larrbuf[ Cpt ];
			VarEdit.YearEdit = (guint)atoi( (Ptr +5) );
			EditYear_alloc_struct();
			BoolAllocStruct = TRUE;
			break;
		}
	}
	
	if( FALSE == BoolAllocStruct ) {
		
		wininfo_create ( WindMain,
			_("Erreur choix de fichier !"),
			_("   Ce fichier ne correspond pas     "),
			  "\n",
			_("   a l'application"),
			  "\n",
			_("   Time Management"),
			  "\n\n",
			_("   Veuillez recommencer."),
			  "\n",
			  "");
	
		g_strfreev( VarEdit.Larrbuf );
		VarEdit.Larrbuf = NULL;
		return( FALSE );
	}
	
	// TEST INTEGRAL DE LA VALIDITE DU FICHIER
	for( Cpt = 0; VarEdit.Larrbuf[ Cpt ]; Cpt++ ) {
	
		// Si commentaire ou ligne vide alors ligne suivante
		if( *VarEdit.Larrbuf[ Cpt ] == '#' || *VarEdit.Larrbuf[ Cpt ] == '\0') continue;

		if( strncmp( VarEdit.Larrbuf[ Cpt ], "DAY", 3 ) == 0 ) {
			// LINE SPLIT
			//	0: DAY
			//	1: MATIN DEBUT
			//	2: MATIN FIN
			//	3: SOIR DEBUT
			//	4: SOIR FIN
			LarrbufLine = g_strsplit( VarEdit.Larrbuf[ Cpt ], " ", 0 );
			
			if( LarrbufLine[ 0 ] && LarrbufLine[ 1 ] && LarrbufLine[ 2 ] && LarrbufLine[ 3 ] && LarrbufLine[ 4 ]  && LarrbufLine[ 5 ] ) {
				BoolAllocStruct = TRUE;
			}
			else {
				BoolAllocStruct = FALSE;
				break;
			}
			
			g_strfreev( LarrbufLine );
			LarrbufLine = NULL;
		}
	}
	
	
	if( FALSE == BoolAllocStruct ) {
		
		wininfo_create ( WindMain,
			_("Fichier corrompu !"),
			_("La structure du fichier est corrompue !"),
			  "\n",
			  "");
	
		g_strfreev( VarEdit.Larrbuf );
		VarEdit.Larrbuf = NULL;
		return( FALSE );
	}
	
	// 
	// YEAR 2012
	// NUMBER_DAY  366
	// 
	// DAY 1 190 690 720 975 Comment
	// DAY 2 191 690 720 975 Comment
	// 
	
	for( Cpt = 0; VarEdit.Larrbuf[ Cpt ]; Cpt++ ) {
		
		// Si commentaire ou ligne vide alors ligne suivante
		if( *VarEdit.Larrbuf[ Cpt ] == '#' || *VarEdit.Larrbuf[ Cpt ] == '\0') continue;

		if( strncmp( VarEdit.Larrbuf[ Cpt ], "DAY", 3 ) == 0 ) {
			
			// LINE SPLIT
			//	0: DAY
			//	1: MATIN DEBUT
			//	2: MATIN FIN
			//	3: SOIR DEBUT
			//	4: SOIR FIN
			LarrbufLine = g_strsplit( VarEdit.Larrbuf[ Cpt ], " ", 0 );
			
			if( LarrbufLine[ 0 ] && LarrbufLine[ 1 ] && LarrbufLine[ 2 ] && LarrbufLine[ 3 ] && LarrbufLine[ 4 ]  && LarrbufLine[ 5 ] ) {
				/*g_print( "1>%s  2>%s  3>%s  4>%s  5>%s  6>%s\n",
					LarrbufLine[ 0 ],	// STRING DAY
					LarrbufLine[ 1 ],	// NUMBER OF DAY
					LarrbufLine[ 2 ],	// MATIN DEBUT
					LarrbufLine[ 3 ],	// MATIN FIN
					LarrbufLine[ 4 ],	// SOIR DEBUT
					LarrbufLine[ 5 ]	// SOIR FIN
					);*/
				
				Day = atoi( LarrbufLine[ 1 ] );
				
				VarEdit.Day[ Day ].MatinDebut = atoi( LarrbufLine[ 2 ] );
				VarEdit.Day[ Day ].MatinFin   = atoi( LarrbufLine[ 3 ] );
				if( VarEdit.Day[ Day ].MatinFin < VarEdit.Day[ Day ].MatinDebut )
					VarEdit.Day[ Day ].MatinFin = VarEdit.Day[ Day ].MatinDebut;
				
				VarEdit.Day[ Day ].SoirDebut  = atoi( LarrbufLine[ 4 ] );
				VarEdit.Day[ Day ].SoirFin    = atoi( LarrbufLine[ 5 ] );
				if( VarEdit.Day[ Day ].SoirFin < VarEdit.Day[ Day ].SoirDebut )
					VarEdit.Day[ Day ].SoirFin = VarEdit.Day[ Day ].SoirDebut;
				
				// IF COMMENT
				// DAY NUMBER_OF_DAY MATIN_DEBUT MATIN_FIN SOIR_DEBUT SOIR_FIN COMMENT...
				// 0   1             2           3         4          5        6
				if( LarrbufLine[ 6 ] ) {
					gint	IndiceComment;
					GString	*StrComment = g_string_new( NULL );
					
					for( IndiceComment = 6; LarrbufLine[ IndiceComment ]; IndiceComment ++ ) {
						g_string_append_printf( StrComment, "%s%s",
									LarrbufLine[ IndiceComment ],
									LarrbufLine[ IndiceComment +1 ] ? " " : ""
									);
					}
					if( NULL != VarEdit.Day[ Day ].Comment ) {
						g_free( VarEdit.Day[ Day ].Comment );
						VarEdit.Day[ Day ].Comment = NULL;
					}
					VarEdit.Day[ Day ].Comment = g_strdup( StrComment->str );
					
					g_string_free( StrComment, TRUE );
					StrComment = NULL;
				}
				else {
					if( NULL != VarEdit.Day[ Day ].Comment ) {
						g_free( VarEdit.Day[ Day ].Comment );
						VarEdit.Day[ Day ].Comment = NULL;
					}
					VarEdit.Day[ Day ].Comment = g_strdup( "" );
				}
			}
			g_strfreev( LarrbufLine );
			LarrbufLine = NULL;
		}
	}
	g_strfreev( VarEdit.Larrbuf );
	VarEdit.Larrbuf = NULL;
	
	return( TRUE );
}
// 
// 
gboolean EditYear_is_modified( void )
{
	return( VarEdit.BoolEdit );
}
// 
// 
void EditYear_set_modified( void )
{
	VarEdit.BoolEdit = TRUE;
}
// 
// 
void EditYear_set_ready( void )
{
	VarEdit.BoolReady = TRUE;
}
// 
// 
void EditYear_set_pathnamefile( gchar *PathNameFile )
{
	if( NULL != Config.PathNameFileBase ) {
		g_free( Config.PathNameFileBase ); 
		Config.PathNameFileBase = NULL;
	}
	Config.PathNameFileBase = g_strdup( PathNameFile );
}
// 
// 
gchar *EditYear_get_namefile( gchar *p_PathNameFile )
{
	// if( FALSE == libutils_test_dir_exist( Config.PathNameFileBase ) ) {
	// 	gchar	*Ptr = strrchr( Config.PathNameFileBase, '/' );
	if( FALSE == libutils_test_dir_exist( p_PathNameFile ) ) {
		gchar	*Ptr = strrchr( p_PathNameFile, '/' );
		Ptr ++;
		return( (gchar *)Ptr );
	}
	return( (gchar *)NULL );
}
// 
// 
void EditYear_set_datas_to_spinbuttons( guint p_DayOfYear )
{
	if( NULL != VarEdit.Day ) {
		gint	H, M;
		
		VarEdit.BoolChanged = FALSE;

		H = VarEdit.Day[ p_DayOfYear ].MatinDebut / 60;
		M = VarEdit.Day[ p_DayOfYear ].MatinDebut % 60;
		gtk_spin_button_set_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_debut_h), H );
		gtk_spin_button_set_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_debut_mn), M );
		
		H = VarEdit.Day[ p_DayOfYear ].MatinFin / 60;
		M = VarEdit.Day[ p_DayOfYear ].MatinFin % 60;
		gtk_spin_button_set_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_h), H );
		gtk_spin_button_set_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_mn), M );
		
		H = VarEdit.Day[ p_DayOfYear ].SoirDebut / 60;
		M = VarEdit.Day[ p_DayOfYear ].SoirDebut % 60;
		gtk_spin_button_set_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_h), H );
		gtk_spin_button_set_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_mn), M );
		
		H = VarEdit.Day[ p_DayOfYear ].SoirFin / 60;
		M = VarEdit.Day[ p_DayOfYear ].SoirFin % 60;
		gtk_spin_button_set_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_fin_h), H );
		gtk_spin_button_set_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_fin_mn), M );
				
		gtk_entry_set_text( GTK_ENTRY(VarCallbacks.Adr_entry_comment), VarEdit.Day[ p_DayOfYear ].Comment );
		
		VarEdit.BoolChanged = TRUE;
	}
}
// 
// 
void EditYear_spinbutton_time_value_changed( GtkSpinButton *spinbutton )
{
	guint	Minutes;
	gint	matin_debut;
	gint	matin_fin;
	gint	soir_debut;
	gint	soir_fin;
	
	if( FALSE == VarEdit.BoolChanged ) return;
	
	if( NULL == VarCallbacks.Adr_spinbutton_matin_debut_h ||
	    NULL == VarCallbacks.Adr_spinbutton_matin_debut_mn ||
	    NULL == VarCallbacks.Adr_spinbutton_matin_fin_h ||
	    NULL == VarCallbacks.Adr_spinbutton_matin_fin_mn ||
	    NULL == VarCallbacks.Adr_spinbutton_soir_debut_h ||
	    NULL == VarCallbacks.Adr_spinbutton_soir_debut_mn ||
	    NULL == VarCallbacks.Adr_spinbutton_soir_fin_h ||
	    NULL == VarCallbacks.Adr_spinbutton_soir_fin_mn ) return;
		
	matin_debut  = (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_debut_h) ) * 60;
	matin_debut += (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_debut_mn) );
	
	matin_fin   = (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_h) ) * 60;
	matin_fin  += (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_mn) );
	
	soir_debut  = (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_h) ) * 60;
	soir_debut += (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_mn) );
	
	soir_fin    = (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_fin_h) ) * 60;
	soir_fin   += (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_fin_mn) );
	
	// 
	// RECTIF DES HEURES ET DES MINUTES :
	// LE TEMPS SUIVANT NE PEUT ETRE INFERIEUR AU TEMPS ACTUEL
	// 	matin_debut  matin_fin  soir_debut  soir_fin
	// 
	if( matin_debut > matin_fin ) {
		Minutes = matin_fin = matin_debut;
	} else {
		Minutes = 0;
	}
	// ACTION SUR HEURE
	gtk_spin_button_set_range(
			GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_h),
			Minutes / 60,
			23
			);
	// ACTION SUR MINUTES
	gtk_spin_button_set_range(
			GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_mn),
			Minutes % 60,
			59
			);

	if( matin_fin > soir_debut ) {
		Minutes = soir_debut = matin_fin;
	} else {
		Minutes = 0;
	}
	// ACTION SUR HEURE
	gtk_spin_button_set_range(
			GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_h),
			Minutes / 60,
			23
			);
	// ACTION SUR MINUTES
	gtk_spin_button_set_range(
			GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_mn),
			Minutes % 60,
			59
			);

	if( soir_debut  > soir_fin ) {
		Minutes = soir_fin = soir_debut;
	} else {
		Minutes = 0;
	}
	// ACTION SUR HEURE
	gtk_spin_button_set_range(
			GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_fin_h),
			Minutes / 60,
			23
			);
	// ACTION SUR MINUTES
	gtk_spin_button_set_range(
			GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_fin_mn),
			Minutes % 60,
			59
			);



	
	
	// AJUSTE LA LIMITE INFERIEURE DE
	// 	MATIN_FIN
	// 	SOIR_DEBUT
	// 	SOIR_FIN
	/*
	gtk_spin_button_set_range(
			GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_h),
			gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_debut_h)),
			23
			);
	gtk_spin_button_set_range(
			GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_h),
			gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_h)),
			23
			);
	gtk_spin_button_set_range(
			GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_fin_h),
			gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_h)),
			23
			);
	*/
	
	// MATIN DEBUT
	if( spinbutton == GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_debut_h) || spinbutton == GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_debut_mn) ) {
		
		Minutes  = (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_debut_h) );
		Minutes *= 60;
		Minutes += (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_debut_mn) );
		VarEdit.Day[ VarEdit.DayOfYear ].MatinDebut = Minutes;
		
		VarEdit.BoolEdit = TRUE;
	}
	
	// MATIN FIN
	else if( spinbutton == GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_h) || spinbutton == GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_mn) ) {
		
		Minutes  = (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_h) );
		Minutes *= 60;
		Minutes += (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_matin_fin_mn) );
		VarEdit.Day[ VarEdit.DayOfYear ].MatinFin = Minutes;		
		
		VarEdit.BoolEdit = TRUE;
	}
	
	// SOIR DEBUT
	else if( spinbutton == GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_h) || spinbutton == GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_mn) ) {
		
		Minutes  = (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_h) );
		Minutes *= 60;
		Minutes += (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_debut_mn) );
		VarEdit.Day[ VarEdit.DayOfYear ].SoirDebut = Minutes;		
		
		VarEdit.BoolEdit = TRUE;
	}
	
	// SOIR FIN
	else if( spinbutton == GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_fin_h) || spinbutton == GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_fin_mn) ) {
		
		Minutes  = (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_fin_h) ) * 60;
		Minutes += (gint)gtk_spin_button_get_value( GTK_SPIN_BUTTON(VarCallbacks.Adr_spinbutton_soir_fin_mn) );
		VarEdit.Day[ VarEdit.DayOfYear ].SoirFin = Minutes;		
		
		VarEdit.BoolEdit = TRUE;
	}
	EditYear_save_struct();
	draw_year();
}
void EditYear_entry_comment_changed( void )
{
	if( FALSE == VarEdit.BoolChanged ) return;
	
	// VarCallbacks.Adr_entry_comment
	if( NULL != VarCallbacks.Adr_entry_comment ) {
		if( NULL != VarEdit.Day[ VarEdit.DayOfYear ].Comment ) {
			g_free( VarEdit.Day[ VarEdit.DayOfYear ].Comment );
			VarEdit.Day[ VarEdit.DayOfYear ].Comment = NULL;
		}
		VarEdit.Day[ VarEdit.DayOfYear ].Comment = g_strdup( (gchar *)gtk_entry_get_text( GTK_ENTRY(VarCallbacks.Adr_entry_comment) ));
		
		VarEdit.BoolEdit = TRUE;
		EditYear_save_struct();	
		draw_year();
	}	
}
// 
// 
void EditYear_set_new_year( gint NewYear )
{
	VarEdit.YearEdit = NewYear;
}
// 
// 
gint EditYear_get_DMY( gint *p_Day, gint *p_Month, gint *p_Year )
{
	GDate	*DateHeap;
	gint	RetDayOfYear;
	
	DateHeap = g_date_new ();
	g_date_set_time_t( DateHeap, time (NULL) );
	*p_Year  = g_date_get_year( DateHeap );
	*p_Month = g_date_get_month( DateHeap );
	*p_Day   = g_date_get_day( DateHeap );
	RetDayOfYear = g_date_get_day_of_year( DateHeap );
	RetDayOfYear --;
	g_date_free( DateHeap );
	
	return( RetDayOfYear );
}
// 
// 
gint EditYear_select_new_year( void )
{
	gint	Day;
	gint	Month;
	gint	Year;
	gint	RetDayOfYear;
	
	RetDayOfYear = EditYear_get_DMY( &Day, &Month, &Year );
	
	// SELECTION ACTUELLE
	if( Year == VarEdit.YearEdit ) {
		gtk_calendar_select_month( GTK_CALENDAR(VarEdit.AdrCalendar), Month -1, VarEdit.YearEdit );
		gtk_calendar_select_day( GTK_CALENDAR(VarEdit.AdrCalendar), Day );
	}
	// SELECTION DEBUT DE L'ANNEE
	else {
		gtk_calendar_select_month( GTK_CALENDAR(VarEdit.AdrCalendar), 0, VarEdit.YearEdit );
		gtk_calendar_select_day( GTK_CALENDAR(VarEdit.AdrCalendar), 1 );
	}
	
	return( RetDayOfYear );
}
// 
// 
guint EditYear_get_new_year( void )
{
	return( VarEdit.YearEdit );
}
// 
// 
void on_calendar_tm_realize( GtkWidget *widget, gpointer user_data )
{
	gint	Year;
	gint	Month;
	gint	Day;
	
	VarEdit.AdrCalendar = widget;
	
	draw_set_adr_call_func_extern( EditYear_draw_page );
	
	EditYear_get_DMY( &Day, &Month, &Year );
	
	gtk_calendar_select_month( GTK_CALENDAR(VarEdit.AdrCalendar), Month -1, Year );
	gtk_calendar_select_day( GTK_CALENDAR(VarEdit.AdrCalendar), Day );
	
	EditYear_set_new_year( Year );
	
	VarEdit.DayAdjust = (VAR_EDIT_DAY *)g_malloc0( sizeof(VAR_EDIT_DAY) * MAX_DAY_ADJUST );

	// LUNDI
	// Config.HoursMonday[ 0 ]
	VarEdit.DayAdjust[ 0 ].MatinDebut = Config.HoursMonday[ 0 ];
	VarEdit.DayAdjust[ 0 ].MatinFin   = Config.HoursMonday[ 1 ];
	VarEdit.DayAdjust[ 0 ].SoirDebut  = Config.HoursMonday[ 2 ];
	VarEdit.DayAdjust[ 0 ].SoirFin    = Config.HoursMonday[ 3 ];
	VarEdit.DayAdjust[ 0 ].Comment    = g_strdup( Config.CommentMonday );
	// MARDI
	// Config.HoursTuesday[ 0 ]
	VarEdit.DayAdjust[ 1 ].MatinDebut = Config.HoursTuesday[ 0 ];
	VarEdit.DayAdjust[ 1 ].MatinFin   = Config.HoursTuesday[ 1 ];
	VarEdit.DayAdjust[ 1 ].SoirDebut  = Config.HoursTuesday[ 2 ];
	VarEdit.DayAdjust[ 1 ].SoirFin    = Config.HoursTuesday[ 3 ];
	VarEdit.DayAdjust[ 1 ].Comment    = g_strdup( Config.CommentTuesday  );
	// MERCREDI
	// Config.HoursWednesday[ 0 ]
	VarEdit.DayAdjust[ 2 ].MatinDebut = Config.HoursWednesday[ 0 ];
	VarEdit.DayAdjust[ 2 ].MatinFin   = Config.HoursWednesday[ 1 ];
	VarEdit.DayAdjust[ 2 ].SoirDebut  = Config.HoursWednesday[ 2 ];
	VarEdit.DayAdjust[ 2 ].SoirFin    = Config.HoursWednesday[ 3 ];
	VarEdit.DayAdjust[ 2 ].Comment    = g_strdup( Config.CommentWednesday );
	// JEUDI
	// Config.HoursThursday[ 0 ]
	VarEdit.DayAdjust[ 3 ].MatinDebut = Config.HoursThursday[ 0 ];
	VarEdit.DayAdjust[ 3 ].MatinFin   = Config.HoursThursday[ 1 ];
	VarEdit.DayAdjust[ 3 ].SoirDebut  = Config.HoursThursday[ 2 ];
	VarEdit.DayAdjust[ 3 ].SoirFin    = Config.HoursThursday[ 3 ];
	VarEdit.DayAdjust[ 3 ].Comment    = g_strdup( Config.CommentThursday );
	// VENDREDI
	// Config.HoursFriday[ 0 ]
	VarEdit.DayAdjust[ 4 ].MatinDebut = Config.HoursFriday[ 0 ];
	VarEdit.DayAdjust[ 4 ].MatinFin   = Config.HoursFriday[ 1 ];
	VarEdit.DayAdjust[ 4 ].SoirDebut  = Config.HoursFriday[ 2 ];
	VarEdit.DayAdjust[ 4 ].SoirFin    = Config.HoursFriday[ 3 ];
	VarEdit.DayAdjust[ 4 ].Comment    = g_strdup( Config.CommentFriday );
	// SAMEDI
	// Config.HoursSaturday[ 0 ]
	VarEdit.DayAdjust[ 5 ].MatinDebut = Config.HoursSaturday[ 0 ];
	VarEdit.DayAdjust[ 5 ].MatinFin   = Config.HoursSaturday[ 1 ];
	VarEdit.DayAdjust[ 5 ].SoirDebut  = Config.HoursSaturday[ 2 ];
	VarEdit.DayAdjust[ 5 ].SoirFin    = Config.HoursSaturday[ 3 ];
	VarEdit.DayAdjust[ 5 ].Comment    = g_strdup( Config.CommentSaturday );
	// DIMANCHE
	// Config.HoursSunday[ 0 ]
	VarEdit.DayAdjust[ 6 ].MatinDebut = Config.HoursSunday[ 0 ];
	VarEdit.DayAdjust[ 6 ].MatinFin   = Config.HoursSunday[ 1 ];
	VarEdit.DayAdjust[ 6 ].SoirDebut  = Config.HoursSunday[ 2 ];
	VarEdit.DayAdjust[ 6 ].SoirFin    = Config.HoursSunday[ 3 ];
	VarEdit.DayAdjust[ 6 ].Comment    = g_strdup( Config.CommentSunday );
}
// 
// 
// 
void on_calendar_tm_day_selected( GtkCalendar *calendar, gpointer user_data )
{
	guint 	year, month, day;
	
	gtk_calendar_get_date( calendar, &year, &month, &day );
	VarEdit.DayOfYear = -1;
	
	if( TRUE == VarEdit.BoolReady && year == VarEdit.YearEdit ) {
		
		GDate	*DateHeap;
	
		// Seeking the number of days for the structure
		DateHeap = g_date_new ();
		g_date_set_year( DateHeap, year );
		g_date_set_month( DateHeap, month +1 );
		g_date_set_day( DateHeap, day );
		VarEdit.DayOfYear = g_date_get_day_of_year( DateHeap );
		// 1 .. 366
		// Adjust the index for the structure
		// g_print( "\t[ 1 .. %d ] = VarEdit.DayOfYear( %d )\n", VarEdit.NbrJoursAnnee, VarEdit.DayOfYear );
		VarEdit.DayOfYear --;
		g_date_free( DateHeap );
		// g_print( "\t[ 0 .. %d ] = VarEdit.DayOfYear( %d )\n", VarEdit.NbrJoursAnnee -1, VarEdit.DayOfYear );

		EditYear_set_datas_to_spinbuttons( VarEdit.DayOfYear );

		EditYear_save_struct();
		draw_year();
	}
	else {
		// WinVte_reset();
		draw_year();
	}
	EditYear_set_flag_buttons();
}
// 
// AFFICHE LE MOIS: p_mont
// 
void EditYear_set_month( gint p_month )
{
	// gboolean gtk_calendar_select_month( GtkCalendar *calendar, guint month, guint year);
	gtk_calendar_select_month( GTK_CALENDAR(VarEdit.AdrCalendar), p_month, VarEdit.YearEdit );
	on_calendar_tm_day_selected( GTK_CALENDAR(VarEdit.AdrCalendar), NULL );

}
// 
// gint p_IndMonth
// 	0=ALL MONTH, 1=JANVIER .. 12=DECEMBRE
// 
void EditYear_apply_weeks( gint p_IndMonth )
{
	gint	MatinDebut;
	gint	MatinFin;
	gint	SoirDebut;
	gint	SoirFin;
	gchar	*Comment = NULL;
	gint	Day;
	gint	DayBegin;
	gint	DayEnd;
		
	// ALL THE DAYS OF YEAR
	if( 0 == p_IndMonth ) {
		
		for( Day = 0; Day < VarEdit.NbrJoursAnnee; Day ++ ) {
			
			// 1=Lundi ... 7=Dimanche
			// VarEdit.Day[ Day ].ThisDay
			EditYear_get_DayAdjust( VarEdit.Day[ Day ].ThisDay -1, &MatinDebut, &MatinFin, &SoirDebut, &SoirFin, &Comment );
			
			VarEdit.Day[ Day ].MatinDebut       = MatinDebut;
			VarEdit.Day[ Day ].MatinFin         = MatinFin;
			VarEdit.Day[ Day ].SoirDebut        = SoirDebut;
			VarEdit.Day[ Day ].SoirFin          = SoirFin;
			if( NULL != VarEdit.Day[ Day ].Comment ) {
				g_free( VarEdit.Day[ Day ].Comment );
				VarEdit.Day[ Day ].Comment = NULL;
			}
			VarEdit.Day[ Day ].Comment          = g_strdup( Comment );
		}
	} else {
		
		EditYear_get_base_days_in_month( p_IndMonth -1, VarEdit.YearEdit, &DayBegin, &DayEnd );
		DayBegin --;
		DayEnd --;
		g_print( "DayBegin= %d\n", DayBegin );
		g_print( "DayEnd  = %d\n", DayEnd );
		for( Day = DayBegin; Day < DayEnd; Day ++ ) {
			
			// 1=Lundi ... 7=Dimanche
			// VarEdit.Day[ Day ].ThisDay
			EditYear_get_DayAdjust( VarEdit.Day[ Day ].ThisDay -1, &MatinDebut, &MatinFin, &SoirDebut, &SoirFin, &Comment );
			
			VarEdit.Day[ Day ].MatinDebut       = MatinDebut;
			VarEdit.Day[ Day ].MatinFin         = MatinFin;
			VarEdit.Day[ Day ].SoirDebut        = SoirDebut;
			VarEdit.Day[ Day ].SoirFin          = SoirFin;
			if( NULL != VarEdit.Day[ Day ].Comment ) {
				g_free( VarEdit.Day[ Day ].Comment );
				VarEdit.Day[ Day ].Comment = NULL;
			}
			VarEdit.Day[ Day ].Comment          = g_strdup( Comment );
		}
	}
	draw_year();
	EditYear_set_datas_to_spinbuttons( VarEdit.DayOfYear );
	EditYear_save_struct();
}
// 
// 
void EditYear_set_DayAdjust( gint p_ind, gint p_MatinDebut, gint p_MatinFin, gint p_SoirDebut, gint p_SoirFin, gchar *p_Comment )
{
	if( -1 != p_MatinDebut ) {
		VarEdit.DayAdjust[ p_ind ].MatinDebut = p_MatinDebut;
	}
	else if( -1 != p_MatinFin ) {
		VarEdit.DayAdjust[ p_ind ].MatinFin   = p_MatinFin;
	}
	else if( -1 != p_SoirDebut ) {
		VarEdit.DayAdjust[ p_ind ].SoirDebut  = p_SoirDebut;
	}
	else if( -1 != p_SoirFin ) {
		VarEdit.DayAdjust[ p_ind ].SoirFin    = p_SoirFin;
	}
	else if( NULL != p_Comment ) {
		if( NULL != VarEdit.DayAdjust[ p_ind ].Comment ) {
			g_free( VarEdit.DayAdjust[ p_ind ].Comment );
			 VarEdit.DayAdjust[ p_ind ].Comment = NULL;
			 VarEdit.DayAdjust[ p_ind ].Comment = g_strdup( p_Comment );
		}
	}
	
	switch( p_ind ) {
	case 0 :
		if( -1 != p_MatinDebut )	Config.HoursMonday[ 0 ] = p_MatinDebut;
		if( -1 != p_MatinFin )		Config.HoursMonday[ 1 ] = p_MatinFin;
		if( -1 != p_SoirDebut )		Config.HoursMonday[ 2 ] = p_SoirDebut;
		if( -1 != p_SoirFin )		Config.HoursMonday[ 3 ] = p_SoirFin;
		if( NULL != Config.CommentMonday ) {
			g_free( Config.CommentMonday );
			Config.CommentMonday = NULL;
		}
		if( NULL != p_Comment )	Config.CommentMonday    = g_strdup( p_Comment );
		else			Config.CommentMonday    = g_strdup( "" );
		break;
	case 1 :
		if( -1 != p_MatinDebut )	Config.HoursTuesday[ 0 ] = p_MatinDebut;
		if( -1 != p_MatinFin )		Config.HoursTuesday[ 1 ] = p_MatinFin;
		if( -1 != p_SoirDebut )		Config.HoursTuesday[ 2 ] = p_SoirDebut;
		if( -1 != p_SoirFin )		Config.HoursTuesday[ 3 ] = p_SoirFin;
		if( NULL != Config.CommentTuesday ) {
			g_free( Config.CommentTuesday );
			Config.CommentTuesday = NULL;
		}
		if( NULL != p_Comment )	Config.CommentTuesday    = g_strdup( p_Comment );
		else			Config.CommentTuesday    = g_strdup( "" );
		break;
	case 2 :
		if( -1 != p_MatinDebut )	Config.HoursWednesday[ 0 ] = p_MatinDebut;
		if( -1 != p_MatinFin )		Config.HoursWednesday[ 1 ] = p_MatinFin;
		if( -1 != p_SoirDebut )		Config.HoursWednesday[ 2 ] = p_SoirDebut;
		if( -1 != p_SoirFin )		Config.HoursWednesday[ 3 ] = p_SoirFin;
		if( NULL != Config.CommentWednesday ) {
			g_free( Config.CommentWednesday );
			Config.CommentWednesday = NULL;
		}
		if( NULL != p_Comment )	Config.CommentWednesday    = g_strdup( p_Comment );
		else			Config.CommentWednesday    = g_strdup( "" );
		break;
	case 3 :
		if( -1 != p_MatinDebut )	Config.HoursThursday[ 0 ] = p_MatinDebut;
		if( -1 != p_MatinFin )		Config.HoursThursday[ 1 ] = p_MatinFin;
		if( -1 != p_SoirDebut )		Config.HoursThursday[ 2 ] = p_SoirDebut;
		if( -1 != p_SoirFin )		Config.HoursThursday[ 3 ] = p_SoirFin;
		if( NULL != Config.CommentThursday ) {
			g_free( Config.CommentThursday );
			Config.CommentThursday = NULL;
		}
		if( NULL != p_Comment )	Config.CommentThursday    = g_strdup( p_Comment );
		else			Config.CommentThursday    = g_strdup( "" );
		break;
	case 4 :
		if( -1 != p_MatinDebut )	Config.HoursFriday[ 0 ] = p_MatinDebut;
		if( -1 != p_MatinFin )		Config.HoursFriday[ 1 ] = p_MatinFin;
		if( -1 != p_SoirDebut )		Config.HoursFriday[ 2 ] = p_SoirDebut;
		if( -1 != p_SoirFin )		Config.HoursFriday[ 3 ] = p_SoirFin;
		if( NULL != Config.CommentFriday ) {
			g_free( Config.CommentFriday );
			Config.CommentFriday = NULL;
		}
		if( NULL != p_Comment )	Config.CommentFriday    = g_strdup( p_Comment );
		else			Config.CommentFriday    = g_strdup( "" );
		break;
	case 5 :
		if( -1 != p_MatinDebut )	Config.HoursSaturday[ 0 ] = p_MatinDebut;
		if( -1 != p_MatinFin )		Config.HoursSaturday[ 1 ] = p_MatinFin;
		if( -1 != p_SoirDebut )		Config.HoursSaturday[ 2 ] = p_SoirDebut;
		if( -1 != p_SoirFin )		Config.HoursSaturday[ 3 ] = p_SoirFin;
		if( NULL != Config.CommentSaturday ) {
			g_free( Config.CommentSaturday );
			Config.CommentSaturday = NULL;
		}
		if( NULL != p_Comment )	Config.CommentSaturday    = g_strdup( p_Comment );
		else			Config.CommentSaturday    = g_strdup( "" );
		break;
	case 6 :
		if( -1 != p_MatinDebut )	Config.HoursSunday[ 0 ] = p_MatinDebut;
		if( -1 != p_MatinFin )		Config.HoursSunday[ 1 ] = p_MatinFin;
		if( -1 != p_SoirDebut )		Config.HoursSunday[ 2 ] = p_SoirDebut;
		if( -1 != p_SoirFin )		Config.HoursSunday[ 3 ] = p_SoirFin;
		if( NULL != Config.CommentSunday ) {
			g_free( Config.CommentSunday );
			Config.CommentSunday = NULL;
		}
		if( NULL != p_Comment )	Config.CommentSunday    = g_strdup( p_Comment );
		else			Config.CommentSunday    = g_strdup( "" );
		break;
	}
}
// 
// 
void EditYear_get_DayAdjust( gint p_ind, gint *p_MatinDebut, gint *p_MatinFin, gint *p_SoirDebut, gint *p_SoirFin, gchar **p_Comment )
{
	*p_MatinDebut = VarEdit.DayAdjust[ p_ind ].MatinDebut;
	*p_MatinFin   = VarEdit.DayAdjust[ p_ind ].MatinFin;
	*p_SoirDebut  = VarEdit.DayAdjust[ p_ind ].SoirDebut;
	*p_SoirFin    = VarEdit.DayAdjust[ p_ind ].SoirFin;
	*p_Comment    = VarEdit.DayAdjust[ p_ind ].Comment;
}


























