 /*
 * file      : fileselect.c
 * project   : tm
 * with      : Gtk-2
 *
 * copyright : (C) 2012 by Claude BULIN
 *
 * TM - Edition des heures de travail / Edition of working hours
 * GNU General Public License
 * 
 * This file is part of TM.
 * 
 * TM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * at your option) any later version.
 * 
 * TM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with TM.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>
#include <string.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_tm/lib.h"
#endif
#include "global.h"
#include "fileselect.h"



/*
typedef enum
{
	GTK_FILE_CHOOSER_ACTION_OPEN,
	GTK_FILE_CHOOSER_ACTION_SAVE,
	GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
	GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER
	
} GtkFileChooserAction;
*/
typedef struct {
	
	GtkWidget		*filechooserdialog;
	GtkFileChooserAction	ChooserAction;
	void			(*Func_Extern) (gchar *string);
	
} VAR_DIALOG;

VAR_DIALOG	VarDialog = {
			NULL,
			FALSE,
			NULL
			};



// 
// fileselect( _("Choix d'un fichier Glade"), GTK_FILE_CHOOSER_ACTION_SAVE | GTK_FILE_CHOOSER_ACTION_OPEN ,  callbacks_choice_file_glade, Path );
// 
void fileselect( gchar *Title, GtkFileChooserAction ChooserAction, void (*Func_Extern) (gchar *string), gchar *Path, GtkWidget *ParentWindow )
{
	GtkFileFilter		*filter;
	
	if (VarDialog.filechooserdialog != NULL) {
		gdk_window_raise (VarDialog.filechooserdialog->window);
		return;
	}
	/*
	typedef enum
	{
	GTK_FILE_CHOOSER_ACTION_OPEN,
	GTK_FILE_CHOOSER_ACTION_SAVE,
	GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
	GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER
	} GtkFileChooserAction;
	
	PRINT_FUNC_LF();
	g_print( "\tPath = %s\n", Path );
	*/
	
	VarDialog.ChooserAction = ChooserAction;
	VarDialog.Func_Extern = Func_Extern;
	
	VarDialog.filechooserdialog = gtk_file_chooser_dialog_new (
							Title,
							NULL,
							ChooserAction,
							GTK_STOCK_CANCEL,
							GTK_RESPONSE_CANCEL,
							GTK_FILE_CHOOSER_ACTION_SAVE == VarDialog.ChooserAction ? GTK_STOCK_SAVE : GTK_STOCK_OPEN,
							GTK_RESPONSE_ACCEPT,
							NULL);
	
	if( GTK_FILE_CHOOSER_ACTION_SAVE == VarDialog.ChooserAction )
		gtk_file_chooser_set_do_overwrite_confirmation( GTK_FILE_CHOOSER (VarDialog.filechooserdialog), TRUE );
	gtk_file_chooser_set_select_multiple( GTK_FILE_CHOOSER (VarDialog.filechooserdialog), FALSE );
	
	if( GTK_FILE_CHOOSER_ACTION_OPEN == VarDialog.ChooserAction || GTK_FILE_CHOOSER_ACTION_SAVE == VarDialog.ChooserAction ) {
		// filter = gtk_file_filter_new ();
		// gtk_file_filter_add_pattern (filter, "*.txt");
		// gtk_file_filter_add_pattern (filter, "*.TXT");
		// gtk_file_filter_set_name (filter, "*.txt *.TXT");
		// gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (VarDialog.filechooserdialog), filter);
	}
	
	gtk_window_set_position (GTK_WINDOW (VarDialog.filechooserdialog), GTK_WIN_POS_CENTER);
	gtk_window_set_type_hint (GTK_WINDOW (VarDialog.filechooserdialog), GDK_WINDOW_TYPE_HINT_DIALOG);
	
	gtk_window_set_transient_for (GTK_WINDOW(VarDialog.filechooserdialog), GTK_WINDOW(ParentWindow));
	gtk_window_set_modal (GTK_WINDOW (VarDialog.filechooserdialog), TRUE);	
	
	libutils_set_default_icone_to_win (VarDialog.filechooserdialog);
	
	if( NULL != Path) {
		
		gchar	*RetPath = NULL;
		gchar	*RetFile = NULL;
		
		libutils_get_path_and_name( Path, &RetPath, &RetFile );
		
		// FOLDER SELECT
		if( NULL != RetPath ) {
			gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER (VarDialog.filechooserdialog), RetPath);
		}
		// FOLDER DEFAULT
		else {
			
		}
		
		if( GTK_FILE_CHOOSER_ACTION_SAVE == VarDialog.ChooserAction ) {
			// NAME_FILE EXIST
			if( NULL != RetFile ) {
				gtk_file_chooser_set_current_name( GTK_FILE_CHOOSER (VarDialog.filechooserdialog), RetFile );
			}
			// NAME_FILE NOT EXIST
			else {
				gtk_file_chooser_set_current_name( GTK_FILE_CHOOSER (VarDialog.filechooserdialog), "BASE" );
			}
		}
		
		if( NULL != RetPath ) {
			g_free( RetPath );
			RetPath = NULL;
		}
		if( NULL != RetFile ) {
			g_free( RetFile );
			RetFile = NULL;
		}
	}
	
	if (gtk_dialog_run (GTK_DIALOG (VarDialog.filechooserdialog)) == GTK_RESPONSE_ACCEPT)
	{
		gchar *filename;
		
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (VarDialog.filechooserdialog));
		// g_print( "FILENAME FROM fileselect() = %s\n", filename);
		if( VarDialog.Func_Extern ) (*VarDialog.Func_Extern) (filename);
		g_free (filename);
	}	
	
	gtk_widget_destroy( VarDialog.filechooserdialog );
	VarDialog.filechooserdialog = NULL;
	VarDialog.Func_Extern = NULL;
}
















